"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppSyncSimulatorConfig = void 0;
const amplify_appsync_simulator_1 = require("amplify-appsync-simulator");
const type_definition_1 = require("amplify-appsync-simulator/lib/type-definition");
const invoke_1 = require("amplify-nodejs-function-runtime-provider/lib/utils/invoke");
const axios_1 = require("axios");
const fs = require("fs");
const lodash_1 = require("lodash");
const merge_graphql_schemas_1 = require("merge-graphql-schemas");
const path = require("path");
const invoke_2 = require("./runtime-providers/java/invoke");
const types_1 = require("./types");
const directLambdaMappingTemplates = {
    request: `{
      "version": "2018-05-29",
      "operation": "Invoke",
      "payload": $utils.toJson($context)
    }`,
    response: `#if($ctx.error)
              $util.error($ctx.error.message, $ctx.error.type, $ctx.result)
             #end
             $util.toJson($ctx.result)`,
};
class AppSyncSimulatorConfig {
    constructor(context, appSyncConfig) {
        this.context = context;
        this.appSyncConfig = appSyncConfig;
        this.mappingTemplatesLocation = path.join(context.serverless.config.servicePath, appSyncConfig.resolversLocation || types_1.DEFAULT_MAPPING_TEMPLATE_LOCATION);
        this.defaultMappingTemplates = appSyncConfig.defaultMappingTemplates;
    }
    getAppSyncConfig() {
        const schemaPaths = Array.isArray(this.appSyncConfig.schema)
            ? this.appSyncConfig.schema
            : [this.appSyncConfig.schema || 'schema.graphql'];
        const schemas = schemaPaths.map((schemaPath) => this.getFileMap(this.context.serverless.config.servicePath, schemaPath));
        const schema = {
            path: (0, lodash_1.first)(schemas).path,
            content: (0, merge_graphql_schemas_1.mergeTypes)(schemas.map((s) => s.content)),
        };
        return {
            appSync: this.makeAppSync(),
            schema,
            resolvers: Object.values(this.appSyncConfig.resolvers).map((mappingTemplate) => this.makeResolver(mappingTemplate, this)),
            dataSources: Object.values(this.appSyncConfig.dataSources)
                .map((datasource) => this.makeDataSource(datasource, this))
                .filter((v) => v !== null),
            functions: Object.values(this.appSyncConfig.pipelineFunctions).map((functionConfiguration) => this.makeFunctionConfiguration(functionConfiguration, this)),
        };
    }
    makeFunctionConfiguration(config, instance) {
        const vtlResponse = this.makeMappingTemplate(config, 'response', instance);
        const vtlRequest = this.makeMappingTemplate(config, 'request', instance);
        return {
            dataSourceName: config.dataSource,
            name: config.name,
            requestMappingTemplateLocation: vtlRequest,
            responseMappingTemplateLocation: vtlResponse,
            requestMappingTemplate: vtlRequest,
            responseMappingTemplate: vtlResponse,
        };
    }
    makeAppSync() {
        return {
            name: this.appSyncConfig.name,
            apiKey: this.context.options.apiKey,
            defaultAuthenticationType: this.makeAuthType(this.appSyncConfig),
            additionalAuthenticationProviders: (this.appSyncConfig.additionalAuthenticationProviders || []).map(this.makeAuthType),
        };
    }
    getMappingTemplate(filePath) {
        return fs.readFileSync(path.join(this.mappingTemplatesLocation, filePath), {
            encoding: types_1.DEFAULT_ENCODING,
        });
    }
    getFileMap(basePath, filePath) {
        return {
            path: filePath,
            content: fs.readFileSync(path.join(basePath, filePath), {
                encoding: types_1.DEFAULT_ENCODING,
            }),
        };
    }
    makeDataSource(source, instance) {
        if (source.name === undefined || source.type === undefined) {
            return null;
        }
        const dataSource = {
            name: source.name,
            type: source.type,
        };
        switch (source.type) {
            case 'AMAZON_DYNAMODB': {
                return {
                    ...dataSource,
                    config: {
                        ...instance.context.options.dynamoDb,
                        tableName: source.config.tableName,
                    },
                };
            }
            case 'AWS_LAMBDA': {
                const functionName = source.name;
                if (functionName === undefined) {
                    instance.logger.log.error(`${source.name} does not have a functionName`, {
                        color: 'orange',
                    });
                    return null;
                }
                const conf = instance.context.options;
                if (conf.functions && conf.functions[functionName] !== undefined) {
                    const func = conf.functions[functionName];
                    return {
                        ...dataSource,
                        invoke: async (payload) => {
                            const result = await axios_1.default.request({
                                url: func.url,
                                method: func.method,
                                data: payload,
                                validateStatus: (a) => false,
                            });
                            return result.data;
                        },
                    };
                }
                const func = this.context.serverless.service.functions[functionName];
                if (func === undefined) {
                    instance.logger.log.error(`The ${functionName} function is not defined`, {
                        color: 'orange',
                    });
                    return null;
                }
                if (func.runtime === 'java8' || func.runtime === 'java11') {
                    return {
                        ...dataSource,
                        invoke: (payload) => (0, invoke_2.invokeResource)({
                            runtime: 'java',
                            srcRoot: path.join(instance.context.serverless.config.servicePath, instance.context.options.location),
                            handler: 'handler' in func ? func.handler : func.name,
                            package: func.environment.servicePackage,
                            event: JSON.stringify(payload),
                            debug: func.environment.debug !== undefined ? func.environment.debug === 'true' : false,
                            envVars: {
                                ...(instance.context.options.lambda.loadLocalEnv === true ? process.env : {}),
                                ...instance.context.serverless.service.provider['environment'],
                                ...func.environment,
                            },
                        }, instance.context),
                    };
                }
                else {
                    return {
                        ...dataSource,
                        invoke: (payload) => (0, invoke_1.invoke)({
                            packageFolder: path.join(instance.context.serverless.config.servicePath, instance.context.options.location),
                            handler: 'handler' in func ? func.handler : func.name,
                            event: JSON.stringify(payload),
                            environment: {
                                ...(instance.context.options.lambda.loadLocalEnv === true ? process.env : {}),
                                ...instance.context.serverless.service.provider['environment'],
                                ...func.environment,
                            },
                        }),
                    };
                }
            }
            case 'AMAZON_ELASTICSEARCH':
            case 'HTTP': {
                // @ts-ignore
                return {
                    ...dataSource,
                    endpoint: source['endpoint'],
                };
            }
            default:
                // @ts-ignore
                return dataSource;
        }
    }
    makeMappingTemplate(config, type, instance) {
        var _a, _b;
        const { name, type: parent, field, substitutions = {} } = config;
        const defaultTemplatePrefix = name || `${parent}.${field}`;
        const templatePath = !(0, lodash_1.isNil)(config === null || config === void 0 ? void 0 : config[type])
            ? config === null || config === void 0 ? void 0 : config[type]
            : !(0, lodash_1.isNil)((_a = instance.defaultMappingTemplates) === null || _a === void 0 ? void 0 : _a[type])
                ? (_b = instance.defaultMappingTemplates) === null || _b === void 0 ? void 0 : _b[type]
                : `${defaultTemplatePrefix}.${type}.vtl`;
        let mappingTemplate;
        // Direct lambda
        // For direct lambdas, we use a default mapping template
        // See https://amzn.to/3ncV3Dz
        if (templatePath === false) {
            mappingTemplate = directLambdaMappingTemplates[type];
        }
        else {
            mappingTemplate = instance.getMappingTemplate(templatePath);
            // Substitutions
            const allSubstitutions = { ...instance.appSyncConfig.substitutions, ...substitutions };
            (0, lodash_1.forEach)(allSubstitutions, (value, variable) => {
                const regExp = new RegExp(`\\$\{?${variable}}?`, 'g');
                mappingTemplate = mappingTemplate.replace(regExp, value);
            });
        }
        mappingTemplate = mappingTemplate.trimStart();
        mappingTemplate = mappingTemplate.trimEnd();
        return mappingTemplate;
    }
    makeResolver(resolver, instance) {
        return {
            kind: resolver.kind,
            fieldName: resolver.field,
            typeName: resolver.type,
            dataSourceName: resolver.kind !== type_definition_1.RESOLVER_KIND.PIPELINE ? resolver === null || resolver === void 0 ? void 0 : resolver.dataSource : '',
            functions: resolver.kind !== type_definition_1.RESOLVER_KIND.UNIT ? resolver === null || resolver === void 0 ? void 0 : resolver.functions : [],
            requestMappingTemplate: this.makeMappingTemplate(resolver, 'request', instance),
            responseMappingTemplate: this.makeMappingTemplate(resolver, 'response', instance),
        };
    }
    makeAuthType(apiConfig) {
        const authType = apiConfig.authentication.type;
        if (authType === amplify_appsync_simulator_1.AmplifyAppSyncSimulatorAuthenticationType.AMAZON_COGNITO_USER_POOLS) {
            return {
                authenticationType: authType,
                cognitoUserPoolConfig: {
                    AppIdClientRegex: apiConfig.authentication.config.AppIdClientRegex,
                },
            };
        }
        else if (authType === amplify_appsync_simulator_1.AmplifyAppSyncSimulatorAuthenticationType.OPENID_CONNECT) {
            return {
                authenticationType: authType,
                openIDConnectConfig: {
                    Issuer: apiConfig.authentication.config.openIDConnectConfig.Issuer,
                    ClientId: apiConfig.authentication.config.openIDConnectConfig.ClientId,
                },
            };
        }
        else if (authType === amplify_appsync_simulator_1.AmplifyAppSyncSimulatorAuthenticationType.API_KEY) {
            return {
                authenticationType: authType,
            };
        }
    }
}
exports.AppSyncSimulatorConfig = AppSyncSimulatorConfig;
//# sourceMappingURL=appsync-simulator-config.js.map