"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JSONUtilities = void 0;
const fs = require("fs-extra");
const path = require("path");
const jsonReader = require("hjson");
/**
 * Wrappers around reading and writing JSON strings
 */
class JSONUtilities {
}
exports.JSONUtilities = JSONUtilities;
JSONUtilities.readJson = (fileName, options) => {
    if (!fileName) {
        throw new Error(`'fileName' argument missing`);
    }
    const mergedOptions = {
        throwIfNotExist: true,
        preserveComments: false,
        ...options,
    };
    if (!fs.existsSync(fileName)) {
        if (mergedOptions.throwIfNotExist) {
            throw new Error(`File at path: '${fileName}' does not exist`);
        }
        else {
            return undefined;
        }
    }
    const content = fs.readFileSync(fileName, 'utf8');
    const data = JSONUtilities.parse(content, {
        preserveComments: mergedOptions.preserveComments,
    });
    return data;
};
JSONUtilities.writeJson = (fileName, data, options) => {
    if (!fileName) {
        throw new Error(`'fileName' argument missing`);
    }
    if (!data) {
        throw new Error(`'data' argument missing`);
    }
    const mergedOptions = {
        minify: false,
        secureFile: false,
        ...options,
    };
    const jsonString = JSONUtilities.stringify(data, {
        minify: mergedOptions.minify,
        orderedKeys: mergedOptions.orderedKeys,
    });
    // Create nested directories if needed
    const dirPath = path.dirname(fileName);
    fs.ensureDirSync(dirPath);
    const writeFileOptions = { encoding: 'utf8', mode: options === null || options === void 0 ? void 0 : options.mode };
    if (mergedOptions.secureFile) {
        writeFileOptions.mode = 0o600;
    }
    fs.writeFileSync(fileName, jsonString, writeFileOptions);
};
JSONUtilities.parse = (jsonString, options) => {
    if (jsonString === undefined || (typeof jsonString === 'string' && jsonString.trim().length === 0)) {
        throw new Error("'jsonString' argument missing or empty");
    }
    const mergedOptions = {
        preserveComments: false,
        ...options,
    };
    let data;
    // By type definition we don't allow any value other than string, but to preserve JSON.parse behavior,
    // which is against the MDN docs we support handling of non-string types like boolean and number.
    if (typeof jsonString === 'string') {
        let cleanString = jsonString;
        // Strip BOM if input has it
        if (cleanString.charCodeAt(0) === 0xfeff) {
            cleanString = cleanString.slice(1);
        }
        data = jsonReader.parse(cleanString, {
            keepWsc: mergedOptions.preserveComments,
        });
    }
    else {
        return jsonString;
    }
    return data;
};
JSONUtilities.stringify = (data, options) => {
    if (!data) {
        throw new Error("'data' argument missing");
    }
    const mergedOptions = {
        minify: false,
        orderedKeys: false,
        ...options,
    };
    let jsonString = '';
    let sortKeys;
    if (mergedOptions.orderedKeys) {
        const allKeys = [];
        // using JSON.stringify to walk the object and push all keys onto a list
        JSON.stringify(data, (k, v) => {
            allKeys.push(k);
            return v;
        });
        sortKeys = allKeys.sort();
    }
    if (mergedOptions.minify) {
        jsonString = JSON.stringify(data, sortKeys);
    }
    else {
        jsonString = JSON.stringify(data, sortKeys, 2);
    }
    return jsonString;
};
//# sourceMappingURL=jsonUtilities.js.map